﻿local NAME, addon = ...
addon.NAME = NAME
XLootGroup = addon

local L = addon.L

local XUI = LibStub('X-UI')
local opt, anchor

local defaults = {
	roll_button_size = 28,
	roll_width = 325,

	roll_anchor_direction = 'up',
	roll_anchor_visible = true,
	roll_anchor_x = UIParent:GetWidth() * .85,
	roll_anchor_y = UIParent:GetHeight() * .6,
}
addon.defaults = defaults
local print = print

--// SKINNING
local Skinner = {}
XUI:SetupSkins(Skinner, {
	row = {},
	item = {},
	anchor = {}
})

local skin_default = {
	name = ('|c2244dd22%s|r'):format('Smooth'),
	texture = [[Interface\AddOns\XLootGroup\textures\border_smooth]],
	bar_texture = [[Interface\AddOns\XLootGroup\textures\bar]],
	color_mod = .85,
	size = 14,
	padding = 1,
}
local skin

local skins_base = {
	anchor = { r = .6, g = .6, b = .6, a = .8 },
	row = { gradient = false },
	item = { backdrop = false }
}

local function CompileSkins()
	local skins = Skinner._skins
	if XLootFrame and XLootFrame.skin then
		skin = XLootFrame.skin
	else
		skin = skin_default
	end
	for which,subskin in pairs(skins) do
		wipe(subskin)
		for k,v in pairs(skin_default) do
			subskin[k] = skin[k]
		end
		for k,v in pairs(skins_base[which]) do
			subskin[k] = v
		end
	end
end

local GetRollFrame, ReskinBars, mouse_focus
local frames = {}
do
	local bars = {}
	function ReskinBars()
		local pad = skin.padding or skin_default.padding
		local tex = skin.bar_texture or skin_default.bar_texture
		local p, n = pad + 3, -pad - 3
		for _,bar in pairs(bars) do
			bar:ClearAllPoints()
			bar:SetPoint('TOPRIGHT', n, n)
			bar:SetPoint('BOTTOMRIGHT', n, p)
			bar:SetPoint('LEFT', bar.parent.icon_frame, 'RIGHT', -pad, 0)
			bar:SetStatusBarTexture(tex)
			local link = bar.parent.link
			if link then
				local r, g, b = GetItemQualityColor(select(3, GetItemInfo(link)))
				bar.parent.overlay:SetBorderColor(r, g, b)
				bar.parent.icon_frame:SetBorderColor(r, g, b)
			end
		end
	end

	local function OnEnter(self)
		mouse_focus = self
		GameTooltip:SetOwner(self.icon_frame, 'ANCHOR_TOPLEFT', 28, 0)
		GameTooltip:SetHyperlink(self.link)
		if IsShiftKeyDown() then
			GameTooltip_ShowCompareItem()
		end
		if IsModifiedClick('DRESSUP') then
			ShowInspectCursor()
		else
			ResetCursor()
		end
	end

	local function OnLeave(self)
		mouse_focus = nil
		GameTooltip:Hide()
	end

	local function OnClick(self, button)
		if IsControlKeyDown() then
			DressUpItemLink(self.link)
		elseif IsShiftKeyDown() then
			ChatEdit_InsertLink(self.link)
		end
	end

	local function BarUpdate(self)
		local now, length = self.expires - GetTime(), self.length
		self.spark:SetPoint('CENTER', self, 'LEFT', (now / length) * self:GetWidth(), 0)
		self:SetValue(now)
	end

	local function OnButtonClick(self)
		RollOnLoot(self.parent.rollid, self.type)
	end
	
	local function Toggle(self, status)
		if status then
			self:Enable()
			self:SetAlpha(1)
		else
			self:Disable()
			self:SetAlpha(.6)
		end
		SetDesaturation(self:GetNormalTexture(), not status)
	end

	local type_index = { 'need', 'greed', 'disenchant', [0] = 'pass' }
	local white = { r = 1, g = 1, b = 1 }
	local function OnButtonEnter(self)
		if self:IsEnabled() == 0 then return end
		mouse_focus = self
		GameTooltip:SetOwner(self, 'ANCHOR_TOPLEFT')
		GameTooltip:SetText(self.label)
		local rtype = type_index[self.type]
		for name,roll in pairs(self.parent.rolls) do
			if roll == rtype then
				local c = RAID_CLASS_COLORS[select(2, UnitClass(name))] or white
				local role = UnitGroupRolesAssigned(name)
				if role ~= 'NONE' then
					name = ('%s (%s)'):format(name, _G[role])
				end
				GameTooltip:AddLine(name, c.r, c.g, c.b)
			end
		end
		GameTooltip:Show()
	end

	local function OnButtonLeave(self)
		GameTooltip:Hide()
	end

	-- TODO: Automatically update tooltip?
	local path = [[Interface\Buttons\UI-GroupLoot-%s-%s]]
	local function CreateRollButton(parent, roll, label, tex, to, x, y)
		local b = CreateFrame('Button', nil, parent)
		b:SetPoint('LEFT', to, 'RIGHT', x, y)
		b:SetWidth(opt.roll_button_size)
		b:SetHeight(opt.roll_button_size)
		b:SetNormalTexture(path:format(tex, 'Up'))
		if tex ~= 'Pass' then
			b:SetHighlightTexture(path:format(tex, 'Highlight'))
			b:SetPushedTexture(path:format(tex, 'Down'))
		else
			b:SetHighlightTexture(path:format(tex, 'Up'))
			b:GetNormalTexture():SetVertexColor(0.8, 0.7, 0.7)
			b:GetHighlightTexture():SetAlpha(0.5)
		end
		b.parent = parent

		local text = b:CreateFontString(nil, 'OVERLAY')
		text:SetFont([[Fonts\FRIZQT__.TTF]], 12, 'THICKOUTLINE')
		text:SetPoint("CENTER", -x + 1, -y)
		b.text = text

		b:SetScript('OnEnter', OnButtonEnter)
		b:SetScript('OnLeave', OnButtonLeave)
		b:SetScript('OnClick', OnButtonClick)
		b:SetMotionScriptsWhileDisabled(true)
		b.Toggle = Toggle
		b.type = roll
		b.label = label

		return b
	end

	function CreateRollFrame()
		local frame = CreateFrame('Button', nil, UIParent)
		frame:SetFrameLevel(anchor:GetFrameLevel())
		frame:SetHeight(24)
		frame:SetWidth(opt.roll_width)
		frame:SetScript('OnEnter', OnEnter)
		frame:SetScript('OnLeave', OnLeave)
		frame:SetScript('OnClick', OnClick)
		frame.OnEnter = OnEnter
		frame.Start = Start
		
		local overlay = CreateFrame('frame', nil, frame)
		overlay:SetFrameLevel(frame:GetFrameLevel())
		overlay:SetAllPoints()
		Skinner:Skin(overlay, 'row')

		local icon_frame = CreateFrame('Frame', nil, frame)
		icon_frame:SetPoint('LEFT', 0, 0)
		icon_frame:SetWidth(28)
		icon_frame:SetHeight(28)
		Skinner:Skin(icon_frame, 'item')

		local icon = icon_frame:CreateTexture(nil, 'BACKGROUND')
		icon:SetPoint('TOPLEFT', 3, -3)
		icon:SetPoint('BOTTOMRIGHT', -3, 3)
		icon:SetTexCoord(.07,.93,.07,.93)
		
		local bar = CreateFrame('StatusBar', nil, frame)
		bar:SetFrameLevel(frame:GetFrameLevel())
		local pad = skin.padding or skin_default.padding
		bar:SetPoint('TOPRIGHT', -pad - 3, -pad - 3)
		bar:SetPoint('BOTTOMRIGHT', -pad - 3, pad + 3)
		bar:SetPoint('LEFT', icon_frame, 'RIGHT', -pad, 0)
		bar:SetStatusBarTexture(skin.bar_texture or skin_default.bar_texture)
		bar:SetScript('OnUpdate', BarUpdate)
		bar.parent = frame
		table.insert(bars, bar)
		
		local spark = bar:CreateTexture(nil, 'OVERLAY')
		spark:SetWidth(14)
		spark:SetHeight(38)
		spark:SetTexture([[Interface\CastingBar\UI-CastingBar-Spark]])
		spark:SetBlendMode('ADD')
		bar.spark = spark

		local bind = icon_frame:CreateFontString(nil, 'OVERLAY')
		bind:SetPoint('BOTTOM', 0, 1)
		bind:SetFont([[Fonts\FRIZQT__.TTF]], 8, 'THICKOUTLINE')
		
		local loot = frame:CreateFontString(nil, 'OVERLAY', 'GameFontNormal')
		loot:SetHeight(16)
		loot:SetJustifyH('LEFT')
		loot:SetPoint('RIGHT', frame, 'RIGHT', -5, 0)

		
		local n = CreateRollButton(frame, 1, NEED, 'Dice', icon_frame, 3, -1)
		local g = CreateRollButton(frame, 2, GREED, 'Coin', n, 0, -2)
		local d = CreateRollButton(frame, 3, ROLL_DISENCHANT, 'DE', g, 0, 2)
		local p = CreateRollButton(frame, 0, PASS, 'Pass', d, 0, 2)

		loot:SetPoint('LEFT', p, 'RIGHT', 3, 0)

		frame.need, frame.greed, frame.disenchant, frame.pass = n, g, d, p
		frame.rolls = {}
		frame.text_bind = bind
		frame.text_loot = loot
		frame.overlay = overlay
		frame.bar = bar
		frame.icon = icon
		frame.icon_frame = icon_frame		
		
		return frame
	end
	
	function GetRollFrame()
		for _, f in ipairs(frames) do
			if not f.rollid then return f end
		end
		
		local f = CreateRollFrame()
		local d = opt.anchor_direction
		if next(frames) then
			local a, b, x, y = 'BOTTOMLEFT', 'TOPLEFT', 0, 4
			if d == 'down' then
				a, b, x, y = 'TOPLEFT', 'BOTTOMLEFT', 0, -4
			elseif d == 'left' then
				a, b, x, y = 'RIGHT', 'LEFT', 4, 0
			elseif d == 'right' then
				a, b, x, y = 'LEFT', 'RIGHT', -4, 0
			end
			f:SetPoint(a, frames[#frames], b, x, y)
		else
			local a, b = 'BOTTOMLEFT', 'TOPLEFT'
			if d == 'down' then
				a, b = 'TOPLEFT', 'BOTTOMLEFT'
			elseif d == 'left' then
				a, b = 'TOPRIGHT', 'BOTTOMRIGHT'
			end
			f:SetPoint(a, anchor, b)
		end
		table.insert(frames, f)
		return f
	end
end

--// EVENT HANDLING
local rolls, over = { }, { }
local function rollevent(event, ...)
	if event == 'selected' then
		local who, item, rtype = ...
		if not rtype then
			print(('XLootGroup error, unknown rtype "%s" in "%s"'):format(rtype, GetLocale()))
		else
			for _,f in pairs(frames) do
				if f.rollid and f.link == item and not f.rolls[who] then
					f.rolls[who] = rtype
					f[rtype].text:SetText((f[rtype].text:GetText() or 0) + 1)
				end
			end
		end
	end
end

local function start(id, length, ongoing)
	if id and over[id] then return end -- Failsafe?
	local f = GetRollFrame()
	
	local icon, name, count, quality, bop, need, greed, de = GetLootRollItemInfo(id)
	local link = GetLootRollItemLink(id)

	local now = length
	if ongoing then
		if quality == 2 then
			length = 60000
		else
			length = 180000
		end
	end

	local bar = f.bar
	length, now = length/1000, now/1000
	bar.length = length
	bar.expires = GetTime() + now

	rolls[id] = f
	f.link = link
	f.rollid = id

	f.need:Toggle(need)
	f.greed:Toggle(greed)
	f.disenchant:Toggle(de)

	f.need.text:SetText()
	f.greed.text:SetText()
	f.pass.text:SetText()
	f.disenchant:SetText()
	
	f.text_bind:SetText(bop and '|cffff4422BoP' or '') -- '|cff44ff44BoE')
	f.text_loot:SetText(name)

	local r, g, b = GetItemQualityColor(quality)
	f.text_loot:SetVertexColor(r, g, b)
	f.overlay:SetBorderColor(r, g, b)
	f.icon_frame:SetBorderColor(r, g, b)
	bar:SetStatusBarColor(r, g, b, .7)
	f.icon:SetTexture(icon)

	bar:SetMinMaxValues(0, length)
	bar:SetValue(now)

	f:Show()

	--local w = GetWatchFrame()

	--w:Start(name, link, quality, icon, length, now)
end

local function cancel(id)
	over[id] = true
	local f = rolls[id]
	if f then
		wipe(f.rolls)
		rolls[id] = nil
		f.rollid = nil
		f.link = nil
		f:Hide()
	end
end

local function modifier(self, modifier, state)
	if mouse_focus and MouseIsOver(mouse_focus) and mouse_focus.OnEnter then
		mouse_focus:OnEnter()
	end
end

--// ADDON SETUP
LibStub('X-Profile'):Setup(addon, 'XLootGroup_Options')
function addon:OnProfileChanged(options)
	opt = options
end

function addon:SkinUpdate()
	CompileSkins()
	Skinner:Reskin()
	ReskinBars()
end

function addon:OnLoad()
	anchor = XUI:CreateAnchor(addon.L.anchor_label, opt, 'roll')
	anchor:SetFrameLevel(5)
	addon.anchor = anchor
	
	anchor:RegisterEvent('START_LOOT_ROLL')
	anchor:RegisterEvent('CANCEL_LOOT_ROLL')
	anchor:RegisterEvent('MODIFIER_STATE_CHANGED')
	anchor:SetScript('OnEvent', function(_, e, ...)
		if e == 'MODIFIER_STATE_CHANGED' then
			modifier(...)
		elseif e == 'START_LOOT_ROLL' then
			start(...)
		elseif e == 'CANCEL_LOOT_ROLL' then
			cancel(...)
		end
	end)

	UIParent:UnregisterEvent("START_LOOT_ROLL")
	UIParent:UnregisterEvent("CANCEL_LOOT_ROLL")

	LibStub('LootEvents'):RegisterGroupCallback(rollevent)
	
	--// XLOOT REGISTRATION
	if XLootFrame and XLootFrame.RegisterPlugin then
		XLootFrame:RegisterPlugin(addon)
	end

	--// SKINNING
	CompileSkins()
	Skinner:Skin(anchor, 'anchor')

	--// ONGOING ROLLS
	if (GetNumPartyMembers() > 0 or GetNumRaidMembers() > 0) and GetLootMethod() == 'group' then
		for i=1,200 do
			local time = GetLootRollTimeLeft(i)
			if time > 0 then
				start(i, time, true)
			end
		end
	end
end

--// SLASH COMMAND
local function option_handler(msg)
	if anchor:IsShown() then
		anchor:Hide()
	else
		anchor:Show()
	end
end
SLASH_XLOOTGROUP1 = '/xlg'
SLASH_XLOOTGROUP2 = '/xlootgroup'
SlashCmdList['XLOOTGROUP'] = option_handler


